# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import enum

from stytch.core.response_base import ResponseBase


class SendRequestLocale(enum.Enum):
    EN = "en"
    ES = "es"
    PTBR = "pt-br"


class SendResponse(ResponseBase):
    """Response type for `Discovery.send`.
    Fields:
    """  # noqa
