# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Optional

from stytch.b2b.models.sso import OIDCConnection
from stytch.core.response_base import ResponseBase


class CreateConnectionResponse(ResponseBase):
    """Response type for `OIDC.create_connection`.
    Fields:
      - connection: The `OIDC Connection` object affected by this API call. See the [OIDC Connection Object](https://stytch.com/docs/b2b/api/oidc-connection-object) for complete response field details.
    """  # noqa

    connection: Optional[OIDCConnection] = None


class UpdateConnectionResponse(ResponseBase):
    """Response type for `OIDC.update_connection`.
    Fields:
      - connection: The `OIDC Connection` object affected by this API call. See the [OIDC Connection Object](https://stytch.com/docs/b2b/api/oidc-connection-object) for complete response field details.
      - warning: If it is not possible to resolve the well-known metadata document from the OIDC issuer, this field will explain what went wrong if the request is successful otherwise. In other words, even if the overall request succeeds, there could be relevant warnings related to the connection update.
    """  # noqa

    connection: Optional[OIDCConnection] = None
    warning: Optional[str] = None
